import { EditorView, type Tooltip } from "@codemirror/view";
import { CompletionTriggerKind } from "vscode-languageserver-protocol";
import type { CompletionContext, CompletionResult } from "@codemirror/autocomplete";
import type { PluginValue, ViewUpdate } from "@codemirror/view";
import type { Transport } from "@open-rpc/client-js/build/transports/Transport";
import type { PublishDiagnosticsParams } from "vscode-languageserver-protocol";
import type * as LSP from "vscode-languageserver-protocol";
interface LSPRequestMap {
    initialize: [LSP.InitializeParams, LSP.InitializeResult];
    "textDocument/hover": [LSP.HoverParams, LSP.Hover];
    "textDocument/completion": [
        LSP.CompletionParams,
        LSP.CompletionItem[] | LSP.CompletionList | null
    ];
    "textDocument/definition": [
        LSP.DefinitionParams,
        LSP.Definition | LSP.DefinitionLink[] | null
    ];
    "textDocument/codeAction": [
        LSP.CodeActionParams,
        (LSP.Command | LSP.CodeAction)[] | null
    ];
    "textDocument/rename": [LSP.RenameParams, LSP.WorkspaceEdit | null];
    "textDocument/prepareRename": [
        LSP.PrepareRenameParams,
        LSP.Range | LSP.PrepareRenameResult | null
    ];
}
interface LSPNotifyMap {
    initialized: LSP.InitializedParams;
    "textDocument/didChange": LSP.DidChangeTextDocumentParams;
    "textDocument/didOpen": LSP.DidOpenTextDocumentParams;
}
interface LSPEventMap {
    "textDocument/publishDiagnostics": LSP.PublishDiagnosticsParams;
}
type Notification = {
    [key in keyof LSPEventMap]: {
        jsonrpc: "2.0";
        id?: null | undefined;
        method: key;
        params: LSPEventMap[key];
    };
}[keyof LSPEventMap];
export declare class LanguageServerClient {
    ready: boolean;
    capabilities: LSP.ServerCapabilities | null;
    initializePromise: Promise<void>;
    private rootUri;
    private workspaceFolders;
    private autoClose?;
    private transport;
    private requestManager;
    private client;
    private plugins;
    constructor(options: LanguageServerClientOptions);
    protected getInitializationOptions(): LSP.InitializeParams["initializationOptions"];
    initialize(): Promise<void>;
    close(): void;
    textDocumentDidOpen(params: LSP.DidOpenTextDocumentParams): Promise<LSP.DidOpenTextDocumentParams>;
    textDocumentDidChange(params: LSP.DidChangeTextDocumentParams): Promise<LSP.DidChangeTextDocumentParams>;
    textDocumentHover(params: LSP.HoverParams): Promise<LSP.Hover>;
    textDocumentCompletion(params: LSP.CompletionParams): Promise<LSP.CompletionItem[] | LSP.CompletionList>;
    textDocumentDefinition(params: LSP.DefinitionParams): Promise<LSP.Definition | LSP.LocationLink[]>;
    textDocumentCodeAction(params: LSP.CodeActionParams): Promise<(LSP.Command | LSP.CodeAction)[]>;
    textDocumentRename(params: LSP.RenameParams): Promise<LSP.WorkspaceEdit>;
    textDocumentPrepareRename(params: LSP.PrepareRenameParams): Promise<LSP.PrepareRenameResult>;
    attachPlugin(plugin: LanguageServerPlugin): void;
    detachPlugin(plugin: LanguageServerPlugin): void;
    protected request<K extends keyof LSPRequestMap>(method: K, params: LSPRequestMap[K][0], timeout: number): Promise<LSPRequestMap[K][1]>;
    protected notify<K extends keyof LSPNotifyMap>(method: K, params: LSPNotifyMap[K]): Promise<LSPNotifyMap[K]>;
    protected processNotification(notification: Notification): void;
}
declare class LanguageServerPlugin implements PluginValue {
    private view;
    private allowHTMLContent;
    client: LanguageServerClient;
    private documentUri;
    private languageId;
    private documentVersion;
    private changesTimeout;
    constructor(view: EditorView, allowHTMLContent?: boolean);
    update({ docChanged }: ViewUpdate): void;
    destroy(): void;
    initialize({ documentText }: {
        documentText: string;
    }): Promise<void>;
    sendChange({ documentText }: {
        documentText: string;
    }): Promise<void>;
    requestDiagnostics(view: EditorView): void;
    requestHoverTooltip(view: EditorView, { line, character }: {
        line: number;
        character: number;
    }): Promise<Tooltip | null>;
    requestCompletion(context: CompletionContext, { line, character }: {
        line: number;
        character: number;
    }, { triggerKind, triggerCharacter, }: {
        triggerKind: CompletionTriggerKind;
        triggerCharacter: string | undefined;
    }): Promise<CompletionResult | null>;
    requestDefinition(view: EditorView, { line, character }: {
        line: number;
        character: number;
    }): Promise<{
        uri: string;
        range: LSP.Range;
    }>;
    processNotification(notification: Notification): void;
    processDiagnostics(params: PublishDiagnosticsParams): Promise<void>;
    private requestCodeActions;
    requestRename(view: EditorView, { line, character }: {
        line: number;
        character: number;
    }): Promise<void>;
}
interface LanguageServerBaseOptions {
    rootUri: string;
    workspaceFolders: LSP.WorkspaceFolder[];
    documentUri: string;
    languageId: string;
}
interface LanguageServerClientOptions extends LanguageServerBaseOptions {
    transport: Transport;
    autoClose?: boolean;
}
interface KeyboardShortcuts {
    rename?: string;
    goToDefinition?: string;
}
interface LanguageServerOptions extends LanguageServerClientOptions {
    client?: LanguageServerClient;
    allowHTMLContent?: boolean;
    keyboardShortcuts?: KeyboardShortcuts;
}
interface LanguageServerWebsocketOptions extends LanguageServerBaseOptions {
    serverUri: `ws://${string}` | `wss://${string}`;
}
export declare function languageServer(options: LanguageServerWebsocketOptions): ({
    extension: import("@codemirror/state").Extension;
} | readonly import("@codemirror/state").Extension[])[];
export declare function languageServerWithTransport(options: LanguageServerOptions): ({
    extension: import("@codemirror/state").Extension;
} | readonly import("@codemirror/state").Extension[])[];
export {};
//# sourceMappingURL=index.d.ts.map