"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.posToOffset = posToOffset;
exports.offsetToPos = offsetToPos;
exports.formatContents = formatContents;
exports.toSet = toSet;
exports.prefixMatch = prefixMatch;
exports.isLSPTextEdit = isLSPTextEdit;
exports.isLSPMarkupContent = isLSPMarkupContent;
exports.showErrorMessage = showErrorMessage;
const marked_1 = require("marked");
function posToOffset(doc, pos) {
    if (pos.line >= doc.lines) {
        return;
    }
    const offset = doc.line(pos.line + 1).from + pos.character;
    if (offset > doc.length) {
        return;
    }
    return offset;
}
function offsetToPos(doc, offset) {
    const line = doc.lineAt(offset);
    return {
        character: offset - line.from,
        line: line.number - 1,
    };
}
function formatContents(contents) {
    if (isLSPMarkupContent(contents)) {
        let value = contents.value;
        if (contents.kind === "markdown") {
            value = (0, marked_1.marked)(value, { async: false });
        }
        return value;
    }
    if (Array.isArray(contents)) {
        return contents.map((c) => `${formatContents(c)}\n\n`).join("");
    }
    if (typeof contents === "string") {
        return contents;
    }
    return "";
}
function toSet(chars) {
    let preamble = "";
    let flat = Array.from(chars).join("");
    const words = /\w/.test(flat);
    if (words) {
        preamble += "\\w";
        flat = flat.replace(/\w/g, "");
    }
    return `[${preamble}${flat.replace(/[^\w\s]/g, "\\$&")}]`;
}
function prefixMatch(items) {
    const first = new Set();
    const rest = new Set();
    for (const item of items) {
        const [initial, ...restStr] = item.textEdit?.newText || item.label;
        first.add(initial);
        for (const char of restStr) {
            rest.add(char);
        }
    }
    const source = `${toSet(first) + toSet(rest)}*$`;
    return [new RegExp(`^${source}`), new RegExp(source)];
}
function isLSPTextEdit(textEdit) {
    return textEdit?.range !== undefined;
}
function isLSPMarkupContent(contents) {
    return contents.kind !== undefined;
}
function showErrorMessage(view, message) {
    const tooltip = document.createElement("div");
    tooltip.className = "cm-error-message";
    tooltip.style.cssText = `
  position: absolute;
  padding: 8px;
  background: #fee;
  border: 1px solid #fcc;
  border-radius: 4px;
  color: #c00;
  font-size: 14px;
  z-index: 100;
  max-width: 300px;
  box-shadow: 0 2px 8px rgba(0,0,0,.15);
`;
    tooltip.textContent = message;
    // Position near the cursor
    const cursor = view.coordsAtPos(view.state.selection.main.head);
    if (cursor) {
        tooltip.style.left = `${cursor.left}px`;
        tooltip.style.top = `${cursor.bottom + 5}px`;
    }
    document.body.appendChild(tooltip);
    // Remove after 3 seconds
    setTimeout(() => {
        tooltip.style.opacity = "0";
        tooltip.style.transition = "opacity 0.2s";
        setTimeout(() => tooltip.remove(), 200);
    }, 3000);
}
//# sourceMappingURL=utils.js.map